/**************************************************************************** 
*  Copyright 2009 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
* m221_ansi.h  
*
*	Header file for the M221 ANSI-C source module.
*
*	C&H Part Number:
*
*****************************************************************************/
#ifndef	M221_HEADER
#define	M221_HEADER

#ifndef PLATFORM
#define		PLATFORM chwin
#endif


#include "apis.h"

/**************************************************************************** 
*  TypeDef's
*****************************************************************************/
typedef char							Char;
typedef unsigned char			UInt8;
typedef char							Int8;
typedef short							Int16;
typedef unsigned short		UInt16;
typedef int								Int32;
typedef unsigned int			UInt32;
typedef unsigned short		Bool;
typedef double						Real64;

typedef struct {
	APIS_HANDLE apis_handle;
	UInt32			apis_error;
	Int16  timeout;  			/***** Settling Time for Relays is Approx 13 ms ****/
	Int16  overlap;
}M221_INSTR, *M221_HANDLE;

/*******************************************************************************
* Error codes
********************************************************************************/
#define		M221_SUCCESS		0

#define		M221_SPECIFIC_ERROR_BASE								0xBFFC0900
#define		M221_ERROR_WAIT_TIMEOUT_EXPIRED					(M221_SPECIFIC_ERROR_BASE + 0)
#define		M221_ERROR_PARAMETER1										(M221_SPECIFIC_ERROR_BASE + 1)
#define		M221_ERROR_PARAMETER2										(M221_SPECIFIC_ERROR_BASE + 2)
#define		M221_ERROR_PARAMETER3										(M221_SPECIFIC_ERROR_BASE + 3)
#define		M221_ERROR_PARAMETER4										(M221_SPECIFIC_ERROR_BASE + 4)
#define		M221_ERROR_FAIL_ID_QUERY								(M221_SPECIFIC_ERROR_BASE + 5)
#define		M221_ERROR_INVALID_CHANNEL_LIST					(M221_SPECIFIC_ERROR_BASE + 6)
#define		M221_ERROR_APIS													(M221_SPECIFIC_ERROR_BASE + 7)
#define		M221_ERROR_OUT_OF_MEMORY								(M221_SPECIFIC_ERROR_BASE + 8)
#define		M221_SPECIFIC_ERROR_END									(M221_SPECIFIC_ERROR_BASE + 9)
#define		M221_SPECIFIC_ERROR_ABSOLUTE_MAX				(M221_SPECIFIC_ERROR_BASE + 0x1FFF)

/**************************************************************************** 
*  Warning Codes
*****************************************************************************/
#define		M221_SPECIFIC_WARNING_BASE							0x3FFC0900
#define		M221_SPECIFIC_WARNING_END								(M221_SPECIFIC_WARNING_BASE + 1)
#define		M221_SPECIFIC_WARNING_ABSOLUTE_MAX			(M221_SPECIFIC_WARNING_BASE + 0x1FFF)

/**************************************************************************** 
*  M-Module ID Words
*****************************************************************************/
#define		M221_IDENT_WORD_SYNC_CODE			     			0
#define		M221_IDENT_WORD_MODULE_NUM				 			1
#define		M221_IDENT_WORD_REVISION_NUM 		   			2
#define		M221_IDENT_WORD_MODULE_CHAR 			 			3
#define		M221_IDENT_WORD_VXI_SYNC_CODE 					16
#define		M221_IDENT_WORD_VXI_ID 					  			17
#define		M221_IDENT_WORD_VXI_DEVICE_TYPE   			18

#define		M221_VXI_MANF_ID												0xFFF
#define		M221_VXI_MODEL_CODE							  			0x25E


/**************************************************************************** 
*	M221 Register offsets
*****************************************************************************/
#define		M221_STATUS_REGISTER										0x00
#define		M221_CONTROL_REGISTER										0x02
#define		M221_INTERRUPT_REGISTER									0x04
#define   M221_RELAY_REGISTER											0x14
#define		M221_ID_ACCESS_REGISTER								  0xFE


/**************************************************************************** 
*	Bit Definitions
*****************************************************************************/
/*  Status Register  */
#define		M221_INTERRUPT_STATUS_BIT								0x01
#define		M221_BUSY_STATUS_BIT										0x80

/*	Control Register */
#define		M221_ENABLE_RELAY_INTERRUPT_BIT					0x0002
#define		M221_RESET_BIT													0x0001

/*   Relay Channels  */
#define		M221_RELAY_CH0_BIT											0x0001
#define		M221_RELAY_CH1_BIT											0x0002
#define		M221_RELAY_CH2_BIT											0x0004
#define		M221_RELAY_CH3_BIT											0x0008
#define		M221_RELAY_CH4_BIT											0x0010
#define		M221_RELAY_CH5_BIT											0x0020
#define		M221_RELAY_CH6_BIT											0x0040
#define		M221_RELAY_CH7_BIT											0x0080

#define   M221_LAST_CH_NUM												7

/*   ID EEPROM       */
#define		M221_DATA_IO_BIT												0x0001   // Value from data out pin of ID EEPROM
#define		M221_CLK_BIT														0x0002	 // Writing a 1 forces SK pin of EEPROM high, 0=low
#define		M221_EEPROM_CHIP_SELECT_BIT							0x0004

/* Used for Get and Set Attribute Functions */
#define  M221_OVERLAP_MODE  0
#define  M221_TIMEOUT  1
#define  M221_INTERRUPT_ENABLE  2

/*****************************************************************
* Misc Defs
******************************************************************/
#define   M221_FALSE      0
#define   M221_TRUE       1
#define   M221_BUSY_TIMEOUT  50 // 50 ms


/**************************************************************************** 
*	Exported Function Prototypes
*****************************************************************************/

Int32 m221_ErrorMessage(Int32 statusCode, Char errorMessage[]);
Int32 m221_RevisionQuery(Int8 driver_rev[]);
Int32 m221_GetAttribute(M221_HANDLE, Int16 attributeName, Int32 *attributeValue);
Int32 m221_GetIdentWord (M221_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 m221_EnableInterrupts (M221_HANDLE handle, Bool enable);
Int32 m221_InterruptStatus(M221_HANDLE handle, Bool *Status);
Int32 m221_ClearInterrupts (M221_HANDLE handle);
Int32 m221_ChannelStatus(M221_HANDLE handle, UInt16 channel, Bool *status);
Int32 m221_DiagTest(M221_HANDLE, Int16 *test_result, Char test_message[]);
Int32 m221_OpcQuery(M221_HANDLE, Int16 *state);
Int32 m221_RouteClose(M221_HANDLE, UInt16 channel);
Int32 m221_RouteCloseList(M221_HANDLE, Char* channelList);
Int32 m221_RouteCloseListQuery(M221_HANDLE, Char* channelList, Char routClosList[]);
Int32 m221_RouteCloseQuery(M221_HANDLE, Int16 channel, Bool *closed);
Int32 m221_RouteOpen(M221_HANDLE, UInt16 channel);
Int32 m221_RouteOpenList(M221_HANDLE, Char* channelList);
Int32 m221_RouteOpenListQuery(M221_HANDLE, Char* channelList, Char routOpenList[]);
Int32 m221_RouteOpenQuery(M221_HANDLE, Int16 channel, Bool *opened);
Int32 m221_SetAll(M221_HANDLE, Bool ch7, Bool ch6, Bool ch5, Bool ch4, Bool ch3, Bool ch2, Bool ch1, Bool ch0);
Int32 m221_SetAllQuery(M221_HANDLE, Bool *ch7, Bool *ch6, Bool *ch5, Bool *ch4, Bool *ch3, Bool *ch2, Bool *ch1, Bool *ch0);
Int32 m221_SetAttribute(M221_HANDLE, Int16 attributeName, Int32 attributeValue);
Int16 m221_WriteWait(M221_HANDLE handle, UInt16 reg, UInt16 data);

/*****************************************************************************
*	Modify the functions below this point to use the MA209 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
Int32 m221_Init(Int32 InstrDesc, Bool id_query, Bool do_reset, M221_HANDLE* vi);
Int32 m221_Close(M221_HANDLE handle);
Int32 m221_Reset(M221_HANDLE handle);
Int32 m221_ReadRegister (M221_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 m221_WriteRegister (M221_HANDLE handle, UInt16 offset, UInt16 value);
Int32 m221_InstallIrq(M221_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr);
Int32 m221_UninstallIrq(M221_HANDLE handle);
Int32 m221_GetApisError (M221_HANDLE handle, UInt32 *apis_error);



#endif		//M221_HEADER
